@@ -2,11 +2,13 @@ import os
 import joblib
 import tempfile
 import shutil
-import numpy as np
 import datetime
 import time
 import math
+import mimetypes
 from typing import Union
+from requests_toolbelt.multipart.encoder import MultipartEncoder
+from requests.exceptions import ConnectionError
 
 from snitch_ai.internal.guid import GUID
 from snitch_ai.internal.api_client import ApiClient
@@ -49,29 +51,53 @@ def _save_model_to_path(model, path):
 
 
 def _save_dataset_to_path(dataset, path):
-    path += ".npz"
-    np.savez_compressed(path, dataset)
+    path += ".joblib"
+    joblib.dump(dataset, path, compress=3)
     return path
 
 
 def _upload_quality_analysis_file(project_id, quality_id, client, file_name, file_path):
     with open(file_path, "rb") as f:
-        resp = client.patch(
-            f"project/{project_id}/quality/{quality_id}", files={file_name: f}
-        )
+        data = _prepare_multipart_file_upload(f, file_name, file_path)
+        try:
+            resp = client.patch(f"project/{project_id}/quality/{quality_id}", data=data, headers={'Content-Type': data.content_type})
+        except ConnectionError:
+            if (os.path.getsize(file_path) > 2147483648):
+                raise Exception(f"Error while uploading files to remote host. Please try uploading files lower than 2GB (current file size: {os.path.getsize(file_path)} bytes).");
+            else:
+                raise Exception("Error while uploading files to remote host. Please try again later.");
+
         if resp.status_code != 200:
             raise Exception(f"Error uploading file: {resp.text}")
 
 
 def _upload_drift_analysis_file(project_id, drift_id, client, file_name, file_path):
     with open(file_path, "rb") as f:
-        resp = client.patch(
-            f"project/{project_id}/drift/{drift_id}", files={file_name: f}
-        )
+        data = _prepare_multipart_file_upload(f, file_name, file_path)
+        try: 
+            resp = client.patch(f"project/{project_id}/drift/{drift_id}", data=data, headers={'Content-Type': data.content_type})
+        except ConnectionError:
+            if (os.path.getsize(file_path) > 2147483648):
+                raise Exception(f"Error while uploading files to remote host. Please try uploading files lower than 2GB (current file size: {os.path.getsize(file_path)} bytes).");
+            else:
+                raise Exception("Error while uploading files to remote host. Please try again later.");
+
         if resp.status_code != 200:
             raise Exception(f"Error uploading file: {resp.text}")
 
 
+def _prepare_multipart_file_upload(file_handle, file_name, file_path) -> MultipartEncoder:
+    mime_type, encoding = mimetypes.guess_type(file_path)
+    if not mime_type:
+        mime_type = "binary/octet-stream"
+
+    return MultipartEncoder(
+        fields={
+            file_name: (os.path.basename(file_path), file_handle, mime_type)
+        }
+    )
+
+
 def _wait_on_analysis_completion(analysis: Union[QualityAnalysis, DriftAnalysis]):
     # wait for analysis to complete
     count = 1
